<?php

namespace Drupal\commerce_product_reminder;

use Drupal\commerce_product\Entity\ProductInterface;
use Drupal\commerce_product\Entity\ProductVariationInterface;
use Drupal\commerce_product_reminder\Entity\ReminderInterface;

/**
 * Interface HelperServiceInterface.
 */
interface HelperServiceInterface {


  /**
   * Get the bundles of an entity type.
   *
   * @param bool $with_label
   * @param string $entity_type
   *
   * @return mixed
   */
  public function getEntityTypeBundles($with_label = TRUE, $entity_type = 'commerce_product');

  /**
   * Get (and set if NULL) the private key.
   */
  public function getPrivateKey();

  /**
   * Get (and set if NULL) the private iv.
   */
  public function getPrivateIv();

  /**
   * Base64 encode a string.
   *
   * @param $data
   *
   * @return string
   */
  public function base64urlEncode($data);

  /**
   * Base64 decode a string.
   *
   * @param $data
   *
   * @return bool|string
   */
  public function base64urlDecode($data);

  /**
   * Encrypt a string.
   *
   * @param $string
   *   The string to encrypt.
   *
   * @return string
   *   The string encrypted.
   */
  public function encrypt($string);

  /**
   * Decrypt a string.
   *
   * @param $string
   *   The string encrypted.
   *
   * @return string
   *   The string decrypted.
   */
  public function decrypt($string);

  /**
   * Generate the hash key used for the unsubscribe link.
   *
   * @param $mail
   * @param $entity_id
   *
   * @return string
   */
  function generateHash($mail, $entity_id);

  /**
   * Load a reminder given a product and an email.
   *
   * @param \Drupal\commerce_product\Entity\ProductInterface $product
   * @param $mail
   *
   * @return \Drupal\commerce_product_reminder\Entity\ReminderInterface|mixed|null
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function loadReminderByProductAndMail(ProductInterface $product, $mail);

  /**
   * Load all reminders, active or not for a product.
   *
   * @param \Drupal\commerce_product\Entity\ProductInterface $product
   *
   * @return \Drupal\commerce_product_reminder\Entity\ReminderInterface[]
   */
  public function loadRemindersByProduct(ProductInterface $product);

  /**
   * Load a Reminder object using the hash.
   *
   * @param $hash
   *
   * @return \Drupal\commerce_product_reminder\Entity\ReminderInterface|mixed|null
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function loadReminderByHash($hash);

  /**
   * Replace token in a string given a product and a product variation.
   *
   * @param $string
   * @param \Drupal\commerce_product\Entity\ProductInterface|NULL $product
   * @param \Drupal\commerce_product\Entity\ProductVariationInterface|NULL $product_variation
   *
   * @return string
   */
  public function replaceToken($string, ProductInterface $product = NULL, ProductVariationInterface $product_variation = NULL);

  /**
   * Get the Reminders entities related to a product.
   *
   * @param \Drupal\commerce_product\Entity\ProductVariationInterface $product_variation
   *
   * @return array|\Drupal\commerce_product_reminder\Entity\ReminderInterface[]
   */
  public function getRemindersFromVariation(ProductVariationInterface $product_variation);

  /**
   * Use cron or not to send emails.
   *
   * @return bool
   */
  public function useCron();

  /**
   * Log is enabled in global settings.
   *
   * @return bool
   */
  public function shouldLog();

  /**
   * Log a reminder deletion.
   *
   * @param \Drupal\commerce_product_reminder\Entity\ReminderInterface $reminder
   *
   */
  public function logDeletion(ReminderInterface $reminder);

  /**
   * Sending mail is enabled ?
   *
   * @return bool
   */
  public function isEnabled();

  /**
   * Get the mail subject.
   *
   * @return string
   */
  public function getSubject();

  /**
   * Get the mail body.
   *
   * @return array
   *   The body array with value and format.
   */
  public function getBody();

  /**
   * Get the mail from email.
   *
   * @return string
   */
  public function getEmailFrom();

  /**
   * Mask a mail address.
   *
   * For example, name@example.org will be masked as n*****@e*****.org.
   *
   * @param $mail
   *   A valid mail address to mask.
   *
   * @return string
   *   The masked mail address.
   */
  function maskEmail($mail);

  /**
   * Send a reminder mail.
   *
   * @param \Drupal\commerce_product\Entity\ProductVariationInterface $product_variation
   * @param \Drupal\commerce_product_reminder\Entity\ReminderInterface $reminder
   *
   * @return bool
   */
  public function sendMail(ProductVariationInterface $product_variation, ReminderInterface $reminder);

}
